/*
www.E2CRE8.be - Brainbox Arduino  - by Bart Huyskens
26/09/2016

Interrupt-based communication bewteen the BBA and a Android smartdevice running a selfmade APPINVENTOR APP
We use the HC06 Bluetooth device
The APP sourcecode and installer are both included in this project and are free to use and alter 

BE AWARE:
That we use a interrupt routine in the microcontroller to be sure that we never miss incoming data.
Arduino IDE is not that great on interrupt handling and that is why we have written some C-code to do the interrupt handling
We use the RX interrupt source of UART1. This generates an interrupt when new data has arrived in UART1 (connected to te HC06 module)

This program
- receives 1 byte from the HC06 module (and so from the app) - to ask for new data - every 100msec (this 100msec is aprox - depending on smartdevice)
- displays that byte (Value 85)on the LCD
- changes the value of led D13 every time a new interrupt occurs
- the uC send 6 bytes - the 6 analog values of the 6 input channels of the BBA - to the app


Connection of HC06 Bluetooth device to Brainbox ARduino:
!!use the 4 pin connector labled 'For HC06' - this fixes all connections for you.
BBA - pin TX    to HC06 pin RXD
BBA - pin RX    to HC06 pin TXD
BBA - +5V       to HC06 pin VCC
BBA - GND       to HC06 pin GND

This program drives an I2C LCD of the type: 
 16 character 2 line I2C Display
 Backpack Interface labelled "YwRobot Arduino LCM1602 IIC V1"  (2€ @ aliexpress)

Connect this LCD as follows:
LCD             Brainbox Arduino
GND             GND
VCC             +5V
SDA             SDA/2
SCL             SCL/3
!! Pull up resistors are optional and not required because ARduino wire lib activates the weak internal pull ups of an AVR - place 4K7 between SDA and 5V and 4K7 between SCL and 5V

To communicate correctly with this I2C LCD you need to install the <LiquidCrystal_I2C.h> library in the arduino IDE
  1- download the "LiquidCrystal_I2C" library as a zip file from https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
  2- do not unzip the file
  3- in ARduino IDE: Sketch >> include library >> add .ZIP library - select the downloaded zip file
  4- this library is installed under 'mydocs'->Arduino : remove it by deleting it there
*/

# include <avr/io.h>
# include <avr/interrupt.h>
# define F_CPU 16000000         // Xtal op 16MHz
# define USART_BAUDRATE 9600
# define BAUD_PRESCALE (((( F_CPU / 16) + ( USART_BAUDRATE / 2) ) / ( USART_BAUDRATE )) - 1)

#include <Wire.h>  // Comes with Arduino IDE
#include <LiquidCrystal_I2C.h> // Use the extra I2C LCD library: https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads

// set the LCD address to 0x27 for a 20 chars 4 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address
 
const byte ledPin = 13;


volatile byte state = LOW;
volatile byte ReceivedByte1;
volatile byte ReceivedByte2;
volatile byte ReceivedByte3;

//Declare the AD variables as int because Arduino always executes a 10 bit AD conversion
int AD_A0 = 0;  
int AD_A1 = 0;  
int AD_A2 = 0;  
int AD_A3 = 0;  
int AD_A4 = 0;  
int AD_A5 = 0;  

// These 8 bit Byte variables are used to downsize the 10 bit AD conversion result into 8 bit
byte ADC_A0 = 0;  
byte ADC_A1 = 0;  
byte ADC_A2 = 0;  
byte ADC_A3 = 0; 
byte ADC_A4 = 0;  
byte ADC_A5 = 0; 

void setup()
{
  pinMode(ledPin, OUTPUT);
  
  lcd.begin(16,2);// initialize LCD library
  lcd.backlight();              //backlight on
  lcd.setCursor(0,0);           // set cursor to positon x=0, y=0
  lcd.print("TEST BT HC06 BBA_03");    // print text on the LCD

  UCSR1B = (1 << RXEN1 ) | (1 << TXEN1 );     // Enable uart1 for transmit and receive
  UCSR1C = (1 << UCSZ10 ) | (1 << UCSZ11 );
  UBRR1H = (BAUD_PRESCALE >> 8);              // Set baud rate register
  UBRR1L = BAUD_PRESCALE;                    // Set baud rate register
  UCSR1B |= (1 << RXCIE1);                   //Enable interrupt when recieve byte is complete
  sei();                                    //Enable global interrupts
}
 
 
void loop()
{
 digitalWrite(ledPin, state);     // flash led D13

AD_A0 = analogRead(A0);              // read the analog value measured at analog pin ANx 
AD_A1 = analogRead(A1);              // read the analog value measured at analog pin ANx 
AD_A2 = analogRead(A2);              // read the analog value measured at analog pin ANx 
AD_A3 = analogRead(A3);              // read the analog value measured at analog pin ANx 
AD_A4 = analogRead(A4);              // read the analog value measured at analog pin ANx 
AD_A5 = analogRead(A5);              // read the analog value measured at analog pin ANx 

ADC_A0 = map(AD_A0, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A1 = map(AD_A1, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A2 = map(AD_A2, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A3 = map(AD_A3, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A4 = map(AD_A4, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A5 = map(AD_A5, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)

//  lcd.clear();                // these LCD lines slow down the whole process a lot. 
//  lcd.setCursor(0,0); 
//  lcd.print("RECEIVING DATA");
//    
//  lcd.setCursor(0,1);
//  lcd.print(ReceivedByte1); 
//  delay(300);
}

// interrupt routine is called when the first byte is received, We know that the second byte will follow imediately after the first one so we read this second byte in the interrupt routine
ISR (USART1_RX_vect)      // interrupt routine that executes every time a new byte is received
{
    state = !state;             // inverse the variable "state" to flash led D13
    UCSR1B &= ~(1 << RXCIE1);   // disable RX interrupt
    ReceivedByte1 = UDR1; // Fetch the received byte value into the variable " ByteReceived " - the RXint flaq is automatically cleared now
    
    while ((UCSR1A & (1 << UDRE1)) == 0) {}; // wait until send buffer is empty
    UDR1 = ADC_A0;                            // send byte from uC via HC06 Bluetooth to App.
    while ((UCSR1A & (1 << UDRE1)) == 0) {}; // wait until send buffer is empty
    UDR1 = ADC_A1;
    while ((UCSR1A & (1 << UDRE1)) == 0) {}; // wait until send buffer is empty
    UDR1 = ADC_A2;
    while ((UCSR1A & (1 << UDRE1)) == 0) {}; // wait until send buffer is empty
    UDR1 = ADC_A3;
    while ((UCSR1A & (1 << UDRE1)) == 0) {}; // wait until send buffer is empty
    UDR1 = ADC_A4;
    while ((UCSR1A & (1 << UDRE1)) == 0) {}; // wait until send buffer is empty
    UDR1 = ADC_A5;
//    while ((UCSR1A & (1 << RXC1)) == 0) {};         // wait until byte 2 is received
//                                                    // Do nothing until data have been received and is ready to be read from UDR 
//    ReceivedByte2 = UDR1;                           // receive byte 2
    
    UCSR1B |= (1 << RXCIE1);        //Re-enable RX interrupt

    
}
